<?xml version="1.0" encoding="UTF-8"?>
<?bluebottle format version="0.1" ?>
<?xml-stylesheet type="text/xsl" href="http://bluebottle.ethz.ch/bluebottle.xsl" ?>
<Text>
<Span style="Normal"><![CDATA[MODULE GUIPatW1;]]></Span><Span style="Comment"><![CDATA[
	(** 	PURPOSE "Template for the construction of a GUI application which can be started only once.
					The template is a complete, correctly working, correctly terminating application";
		TECHNIQUE "An anchor point winstance is either NIL or points to a window";
		USAGE  "1 - Make a copy of this template and assign it a name of your choice";
				"2 - In the copy, replace all occurences of  "GUIPatW1" by the name of the new module";
		AUTHOR "afi";
	 *)]]></Span><Span style="Normal"><![CDATA[

IMPORT
	AosModules, WMComponents, WM := WMWindowManager, WMStandardComponents,
	Utilities
	;

TYPE
	]]></Span><Span style="Bold"><![CDATA[Window* = OBJECT (WMComponents.FormWindow)]]></Span><Span style="Normal"><![CDATA[

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE CreateForm () : WMComponents.VisualComponent;]]></Span><Span style="Normal"><![CDATA[
		VAR panel : WMStandardComponents.Panel;
		BEGIN
			NEW (panel); panel.bounds.SetExtents (600, 400); panel.fillColor.Set (0FFFFFFFFH);
			
			RETURN panel
		END CreateForm;

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE &New ();	(* A *)
]]></Span><Span style="Normal"><![CDATA[		VAR vc : WMComponents.VisualComponent;]]></Span><Span style="Comment"><![CDATA[
]]></Span><Span style="Normal"><![CDATA[		BEGIN
			vc := CreateForm ();
			Init (vc.bounds.GetWidth (), vc.bounds.GetHeight (), FALSE);
			SetContent (vc);
			WM.DefaultAddWindow (SELF);			
			SetTitle (Utilities.NewString ("GUIPatW1"));		
		END New;

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE Close ();	(* B *)]]></Span><Span style="Normal"><![CDATA[
		BEGIN

			Close^;
			winstance := NIL
		END Close;
		
	END Window;

VAR
	winstance : Window;

]]></Span><Span style="Comment"><![CDATA[(** Standard single window, non-restorable. *)]]></Span><Span style="Normal"><![CDATA[
]]></Span><Span style="Bold"><![CDATA[PROCEDURE Open* (par : ANY) : ANY;]]></Span><Span style="Normal"><![CDATA[
BEGIN
	IF winstance = NIL THEN NEW (winstance) END;
	RETURN NIL
END Open;
]]></Span><Span style="Bold"><![CDATA[
PROCEDURE Cleanup;
]]></Span><Span style="Normal"><![CDATA[BEGIN]]></Span><Span style="Bold"><![CDATA[
]]></Span><Span style="Normal"><![CDATA[	IF winstance # NIL THEN winstance.Close END
END Cleanup;

BEGIN
	AosModules.InstallTermHandler (Cleanup);

	]]></Span><Span style="Comment"><![CDATA[(* Additional BEGIN processing. *)]]></Span><Span style="Normal"><![CDATA[
END GUIPatW1.

GUIPatW1.Open ~
S.Free GUIPatW1 ~

]]></Span><Span style="Bold"><![CDATA[Comments]]></Span><Span style="Normal"><![CDATA[

A.	&New is a constructor of an instance of Window, here winstance.

B.	winstance.Close (found in Cleanup) removes the window from the display space and
	because it is so redefined, winstance is set to NIL.
	By omitting this redefinition PROCEDURE Close in (* B*), the window would be removed
	but the pointer in winstance would remain unchanged. An instance of an object can not
	access and change pointers that point to the instance: SELF := NIL has no effect.
	One cannot say that winstance.Close is a "deconstructor" which has a special meaning in
	object oriented systems.
]]></Span>

</Text>
