<?xml version="1.0" encoding="UTF-8"?>
<?bluebottle format version="0.1" ?>
<?xml-stylesheet type="text/xsl" href="http://bluebottle.ethz.ch/bluebottle.xsl" ?>
<Text>
<Span style="Normal"><![CDATA[MODULE GUIPatWn;]]></Span><Span style="Comment"><![CDATA[
	(** 	PURPOSE "Template for the construction of a GUI application which can be started multiple times.
					The template is a complete, correctly working, correctly terminating application";
		TECHNIQUE "The number of opened windows is counted. At cleanup time, a killer message is broadcast";
		USAGE  "1 - Make a copy of this template and assign it a name of your choice";
				"2 - In the copy, replace all occurences of  "GUIPatWn" by the name of the new module";
		AUTHOR "afi";
*)]]></Span><Span style="Normal"><![CDATA[

IMPORT
	AosModules, WMComponents, WM := WMWindowManager, WMMessages, WMStandardComponents,
	Utilities
	;

TYPE
	]]></Span><Span style="Bold"><![CDATA[Window* = OBJECT (WMComponents.FormWindow)]]></Span><Span style="Normal"><![CDATA[
	

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE CreateForm () : WMComponents.VisualComponent;]]></Span><Span style="Normal"><![CDATA[
		VAR panel : WMStandardComponents.Panel;
		BEGIN
			NEW (panel); panel.bounds.SetExtents (600, 400); panel.fillColor.Set (0FFFFFFFFH);
			
			RETURN panel
		END CreateForm;

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE &New ();
]]></Span><Span style="Normal"><![CDATA[		VAR vc : WMComponents.VisualComponent;]]></Span><Span style="Comment"><![CDATA[
]]></Span><Span style="Normal"><![CDATA[		BEGIN
			IncCount;
			vc := CreateForm ();
			Init (vc.bounds.GetWidth (), vc.bounds.GetHeight (), FALSE);
			SetContent (vc);
			
			WM.DefaultAddWindow (SELF);
			SetTitle (Utilities.NewString ("GUIPatWn"));		
		END New;

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE Close;]]></Span><Span style="Normal"><![CDATA[
		BEGIN

			Close^;
			DecCount
		END Close;

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE Handle (VAR x: WMMessages.Message);]]></Span><Span style="Normal"><![CDATA[
		BEGIN
			IF (x.msgType = WMMessages.MsgExt) & (x.ext # NIL) & (x.ext IS KillerMsg) THEN Close
			ELSE Handle^ (x)
			END
		END Handle;			

	END Window;

	]]></Span><Span style="Bold"><![CDATA[KillerMsg = OBJECT
]]></Span><Span style="Normal"><![CDATA[	END KillerMsg; 	

VAR
	nofWindows : LONGINT;

]]></Span><Span style="Comment"><![CDATA[(** Standard multiple windows controlled by number, non-restorable. *)]]></Span><Span style="Bold"><![CDATA[
PROCEDURE Open* (par : ANY) : ANY;
]]></Span><Span style="Normal"><![CDATA[VAR winstance : Window;
BEGIN
	NEW (winstance);
	RETURN NIL
END Open;
]]></Span><Span style="Bold"><![CDATA[
PROCEDURE IncCount;]]></Span><Span style="Normal"><![CDATA[
BEGIN {EXCLUSIVE}
	INC (nofWindows);
END IncCount;		

]]></Span><Span style="Bold"><![CDATA[PROCEDURE DecCount;]]></Span><Span style="Normal"><![CDATA[
BEGIN {EXCLUSIVE}
	DEC (nofWindows);
END DecCount;	
]]></Span><Span style="Bold"><![CDATA[
PROCEDURE Cleanup;
]]></Span><Span style="Normal"><![CDATA[VAR die : KillerMsg;
	 msg : WMMessages.Message;
	 m : WM.WindowManager;
BEGIN {EXCLUSIVE}
	NEW (die);
	msg.ext := die;
	msg.msgType := WMMessages.MsgExt;
	m := WM.GetDefaultManager ();
	m.Broadcast (msg);
	AWAIT (nofWindows = 0)
END Cleanup;

BEGIN
	AosModules.InstallTermHandler (Cleanup);

	]]></Span><Span style="Comment"><![CDATA[(* Additional BEGIN processing. *)]]></Span><Span style="Normal"><![CDATA[
END GUIPatWn.

GUIPatWn.Open ~
S.Free GUIPatWn ~

]]></Span>

</Text>
