<?xml version="1.0" encoding="UTF-8"?>
<?bluebottle format version="0.1" ?>
<?xml-stylesheet type="text/xsl" href="http://bluebottle.ethz.ch/bluebottle.xsl" ?>
<Text>
<Span style="Normal"><![CDATA[MODULE GUIPatWnR;]]></Span><Span style="Comment"><![CDATA[
	(** 	PURPOSE "Template for the construction of a restorable GUI application which can be started multiple times.
					The template is a complete, correctly working, correctly terminating application";
		TECHNIQUE "The number of opened windows is counted. At cleanup time, a killer message is broadcast";
		USAGE  "1 - Make a copy of this template and assign it a name of your choice";
				"2 - In the copy, replace all occurences of  "GUIPatWnR" by the name of the new module";
				"3 - Remove these comments and add comments about the new application to develop";
				"4 - Blue text denotes, by an unwritten convention, debugging statements (see Text Styles),
					but here additions required for restoring an application appear in blue.";
		AUTHOR "afi";
	 *)]]></Span><Span style="Normal"><![CDATA[

IMPORT
	AosModules, WMComponents,WM := WMWindowManager, WMMessages, WMStandardComponents,
	Utilities]]></Span><Span style="Debug"><![CDATA[, WMRestorable]]></Span><Span style="Normal"><![CDATA[
	;

TYPE
	]]></Span><Span style="Bold"><![CDATA[Window* = OBJECT (WMComponents.FormWindow)]]></Span><Span style="Normal"><![CDATA[

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE CreateForm () : WMComponents.VisualComponent;]]></Span><Span style="Normal"><![CDATA[
		VAR panel : WMStandardComponents.Panel;
		BEGIN
			NEW (panel); panel.bounds.SetExtents (600, 400); panel.fillColor.Set (0FFFFFFFFH);
			
			RETURN panel
		END CreateForm;

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE &New (c : WMRestorable.Context);
]]></Span><Span style="Normal"><![CDATA[		VAR vc : WMComponents.VisualComponent;]]></Span><Span style="Comment"><![CDATA[
]]></Span><Span style="Normal"><![CDATA[		BEGIN
			IncCount;
			vc := CreateForm ();
			Init (vc.bounds.GetWidth (), vc.bounds.GetHeight (), FALSE);
			SetContent (vc);
]]></Span><Span style="Debug"><![CDATA[			IF c # NIL THEN 
				(* restore the desktop *)
				WMRestorable.AddByContext (SELF, c, {WM.FlagFrame});
				IF c.appData # NIL THEN
					(* Insert application code *)
				END			
			ELSE 
]]></Span><Span style="Normal"><![CDATA[				WM.DefaultAddWindow (SELF)			
]]></Span><Span style="Debug"><![CDATA[			END;
]]></Span><Span style="Normal"><![CDATA[			SetTitle (Utilities.NewString ("GUIPatWnR"));		
		END New;

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE Close;]]></Span><Span style="Normal"><![CDATA[
		BEGIN

			Close^;
			DecCount
		END Close;
		
]]></Span><Span style="Debug"><![CDATA[		PROCEDURE Handle (VAR x: WMMessages.Message);
		BEGIN
			IF (x.msgType = WMMessages.MsgExt) & (x.ext # NIL) THEN
				IF (x.ext IS KillerMsg) THEN Close
				ELSIF (x.ext IS WMRestorable.Storage) THEN
					x.ext (WMRestorable.Storage).Add ("GUIPatWnR", "GUIPatWnR.Restore", SELF, NIL)
				ELSE Handle^ (x)
				END
			ELSE Handle^ (x)
			END
		END Handle;			
]]></Span><Span style="Normal"><![CDATA[
	END Window;

	]]></Span><Span style="Bold"><![CDATA[KillerMsg = OBJECT
]]></Span><Span style="Normal"><![CDATA[	END KillerMsg; 	

VAR
	nofWindows : LONGINT;

]]></Span><Span style="Comment"><![CDATA[(** Standard multiple windows controlled by number, restorable. *)]]></Span><Span style="Bold"><![CDATA[
PROCEDURE Open* (par : ANY) : ANY;
]]></Span><Span style="Normal"><![CDATA[VAR winstance : Window;
BEGIN
	NEW (winstance, NIL);
	RETURN NIL
END Open;
]]></Span><Span style="Bold"><![CDATA[
]]></Span><Span style="Debug"><![CDATA[PROCEDURE Restore* (par : ANY) : ANY;
VAR winstance : Window;
BEGIN
	IF (par # NIL) & (par IS WMRestorable.Context) THEN
		NEW (winstance, par (WMRestorable.Context))
	END;
	RETURN NIL
END Restore;
]]></Span><Span style="Bold"><![CDATA[
PROCEDURE IncCount;]]></Span><Span style="Normal"><![CDATA[
BEGIN {EXCLUSIVE}
	INC (nofWindows);
END IncCount;		

]]></Span><Span style="Bold"><![CDATA[PROCEDURE DecCount;]]></Span><Span style="Normal"><![CDATA[
BEGIN {EXCLUSIVE}
	DEC (nofWindows);
END DecCount;	
]]></Span><Span style="Bold"><![CDATA[
PROCEDURE Cleanup;
]]></Span><Span style="Normal"><![CDATA[VAR die : KillerMsg;
	 msg : WMMessages.Message;
	 m : WM.WindowManager;
BEGIN {EXCLUSIVE}
	NEW (die);
	msg.ext := die;
	msg.msgType := WMMessages.MsgExt;
	m := WM.GetDefaultManager ();
	m.Broadcast (msg);
	AWAIT (nofWindows = 0)
END Cleanup;

BEGIN
	AosModules.InstallTermHandler (Cleanup);

	]]></Span><Span style="Comment"><![CDATA[(* Additional BEGIN processing.*)]]></Span><Span style="Normal"><![CDATA[
END GUIPatWnR.

GUIPatWnR.Open ~
S.Free GUIPatWnR ~

]]></Span>

</Text>
